import numpy as np
import os
import imageio
from os.path import basename,splitext
import matplotlib.pyplot as plt 
import SimpleITK as sitk 
import scipy 
import sys

from library_dicom.dicom_processor.model.reader.Instance import Instance
from library_dicom.dicom_processor.model.NiftiBuilder import NiftiBuilder
from library_dicom.dicom_processor.enums.TagEnum import *
from library_dicom.dicom_processor.enums.SopClassUID import *

class Series():
    """ A class representing a series Dicom
    """
    
    def __init__(self, path):
        """Construct a Dicom Series Object

        Arguments:
            path {String} -- [Absolute Path where Dicom Series is located (hirachical Dicoms)]
        """

        self.path = path
        self.file_names = os.listdir(path)
        self.number_of_files = len(self.file_names)

    def get_number_of_files(self):
        return self.number_of_files
    
    def get_first_instance_metadata(self):
        firstFileName = self.file_names[0]
        return Instance(os.path.join(self.path,firstFileName), load_image=True)


    def get_series_details(self):
        """Read the first dicom in the folder and store Patient / Study / Series
        informations

        Returns:
            [dict] -- [Return the details of a Serie from the first Dicom]
        """
        self.series_details = {}
        self.patient_details = {}
        self.study_details = {}

        dicomInstance = self.get_first_instance_metadata()

        self.series_details = dicomInstance.get_series_tags()
        self.patient_details = dicomInstance.get_patients_tags()
        self.study_details = dicomInstance.get_studies_tags()
        self.instance_details = dicomInstance.get_instance_tags()
        self.sop_class_uid = dicomInstance.get_sop_class_uid()
        self.is_image_series = dicomInstance.is_image_modality()

        return {
            'series' : self.series_details,
            'study' : self.study_details,
            'patient' : self.patient_details,
            'path' : self.path,
            'files' : self.number_of_files,
            'instance' : self.instance_details
        }

    def is_series_valid(self):
        firstDicomDetails = self.get_series_details()
        #Le tag number of slice n'est que pour PT et NM, pas trouvé d'autre tag fiable pour les autres series
        if firstDicomDetails['series']['NumberOfSlices']!="Undefined" and firstDicomDetails['series']['NumberOfSlices'] != len(self.file_names):
            print('wrong number of slice')
            return False

        for fileName in self.file_names:
            dicomInstance = Instance(os.path.join(self.path, fileName), load_image=False)
            if (dicomInstance.get_series_tags()['SeriesInstanceUID'] != firstDicomDetails['series']['SeriesInstanceUID']):
                print('Not same Series Instance UID')
                return False
        
        return True
        
    def is_image_modality(self):
        return self.get_first_instance_metadata().is_image_modality()

    def is_primary_image(self):
        return ( "PRIMARY" in self.series_details['ImageType'])

    def is_localizer_series(self):
        return ( "LOCALIZER" in self.series_details['ImageType'])


    def get_instances_ordered(self):
        instance_array = [Instance(os.path.join(self.path, file_name), load_image=True) for file_name in self.file_names]
        instance_array.sort(key=lambda instance_array:int(instance_array.get_image_position()[2]))
        self.instance_array = instance_array
        return instance_array 


    def get_numpy_array(self):
        if self.is_image_modality == False : return

        #instance_array = [Instance(os.path.join(self.path, file_name), load_image=True) for file_name in self.file_names]
        #instance_array.sort(key=lambda instance_array:int(instance_array.get_image_position()[2]))
        pixel_data = [instance.get_image_nparray() for instance in self.instance_array]
        np_array = np.stack(pixel_data,axis=-1)
        #A VERIF
        #self.instance_array = instance_array

        return np_array


    

    def get_z_positions(self):
        Z_positions = [ instance.get_image_position()[2] for instance in self.instance_array ]
        return Z_positions

    
    def get_z_spacing(self):
        """ called by __getMetadata """
        Z_positions = [ instance.get_image_position()[2] for instance in self.instance_array ]
        #print(Z_positions)

        initial_z_spacing = round(abs(Z_positions[0] - Z_positions[1]), 1)
        for i in range(2,len(Z_positions)):

            z_spacing = round(abs(Z_positions[i - 1] - Z_positions[i]), 1)
            if z_spacing < initial_z_spacing - float(0.1) or z_spacing > initial_z_spacing + float(0.1) :
                try : 
                    raise Exception('Unconstant Spacing')
                except Exception : 
                    return('Unconstant Spacing') #alerte #return
        return np.mean(self.calculate_z_spacing(round_=False))


    def calculate_z_spacing(self, round_): 
        Z_positions = [ instance.get_image_position()[2] for instance in self.instance_array ]
        spacing = []

        if round_ == False : 
            initial_z_spacing = Z_positions[0] - Z_positions[1]
            spacing.append(initial_z_spacing)
            for i in range(2,len(Z_positions)):
                z_spacing = Z_positions[i - 1] - Z_positions[i]
                spacing.append(z_spacing)

            return spacing

        else : 
            #print(abs(Z_positions[0] - Z_positions[1]))
            initial_z_spacing = round(abs(Z_positions[0] - Z_positions[1]), 1)
            spacing.append(initial_z_spacing)
            for i in range(2,len(Z_positions)):
                z_spacing = round(abs(Z_positions[i - 1] - Z_positions[i]), 1)
                spacing.append(z_spacing)  

            return spacing 



    #check origin direction spacing de nifti

    def export_nifti(self, file_path, mask = None):
        if (mask is None) : 
            nifti_builder = NiftiBuilder(self)
            nifti_builder.save_nifti(file_path)
        else : 
            nifti_builder = NiftiBuilder(self)
            nifti_builder.save_nifti(file_path, mask)

    def get_all_SOPInstanceIUD(self):
        liste = []
        for filename in self.file_names : 
            instanceData = Instance(os.path.join(self.path,filename), load_image=True)
            liste.append(instanceData.get_SOPInstanceUID())
        return liste 


    def get_all_acquisition_time(self):
        liste = []
        for filename in self.file_names : 
            instanceData = Instance(os.path.join(self.path,filename), load_image=True)
            liste.append(instanceData.get_acquisition_time())
        return sorted(liste)



    def get_size_matrix(self):
        size = []
        data = self.get_first_instance_metadata()
        x = data.get_number_rows()
        size.append(x)
        y = data.get_number_columns()
        size.append(y)
        z = len(self.get_all_SOPInstanceIUD())
        size.append(z)
        return size



    


import numpy as np
import SimpleITK as sitk 
from skimage.measure import label
#from radiomics.featureextractor import RadiomicsFeatureExtractor
import matplotlib.pyplot as plt
import scipy as sc
from sklearn import mixture
from library_dicom.post_processing.PostProcess_Reader import PostProcess_Reader

class GaussianModel(PostProcess_Reader) :


    def __init__(self, mask_path, pet_path, type) : 
        super().__init__(mask_path, pet_path, type)
        self.parameters = self.gaussien_mixture_model()


    def gaussien_mixture_model(self) : 
        parameters = {}
        for key in range(1, len(self.features) + 1) : 
            subdict = {}
            X_train = np.asarray(self.features[key]['suv_values']).reshape((-1,1))
            #si volume plus petit que 30ml, on laisse
            if self.features[key]['volume'] < float(30) : 

                lowest_bic = np.infty
                bic = []
                for cv_type in cv_types : 
                    gmm = mixture.GaussianMixture(n_components= 1, covariance_type=cv_type, random_state = 0)
                    gmm.fit(X_train)
                    bic.append(gmm.bic(X_train))
                    if bic[-1] < lowest_bic : 
                        lowest_bic = bic[-1]
                        best_gmm = gmm 
                        best_n_components = 1
                        best_cv_type = cv_type
                        labels = best_gmm.predict(X_train) #ndarray

                subdict['best_n_components'] = best_n_components
                subdict['best_cv_type'] = best_cv_type
                subdict['label'] = labels

                parameters[key] = subdict

            else : 

                #find best parameters for model 

                lowest_bic = np.infty
                bic = []
                n_components_range = range(1, 8)
                cv_types = ['spherical', 'tied', 'diag', 'full']
                for cv_type in cv_types : 
                    for n_components in n_components_range : 
                        gmm = mixture.GaussianMixture(n_components=n_components, covariance_type=cv_type, random_state = 0)
                        gmm.fit(X_train)
                        bic.append(gmm.bic(X_train))
                        if bic[-1] < lowest_bic : 
                            lowest_bic = bic[-1]
                            best_gmm = gmm 
                            best_n_components = n_components
                            best_cv_type = cv_type
                            labels = best_gmm.predict(X_train) #ndarray
                            
                subdict['best_n_components'] = best_n_components
                subdict['best_cv_type'] = best_cv_type
                subdict['label'] = labels

                parameters[key] = subdict

        return parameters


    def get_histogram_best_model(self, label):
        X_train = np.asarray(self.features[label]['suv_values']).reshape((-1,1))
        n_components = self.parameters[label]['best_n_components']
        cv_type = self.parameters[label]['best_cv_type']

        gmm = mixture.GaussianMixture(n_components=n_components, covariance_type=cv_type, random_state = 0)
        gmm.fit(X_train)

        gmm_x = np.linspace(0, np.max(X_train), X_train.shape[0])
        gmm_y = np.exp(gmm.score_samples(gmm_x.reshape(-1,1)))

        plt.hist(self.features[label]['suv_values'], bins = 'fd', normed = True, alpha = 0.6, label = 'suv_value')
        plt.xlabel('suv_value')
        plt.ylabel('frequency')
            
        plt.plot(gmm_x, gmm_y, label = 'gmm model', color = 'mediumvioletred', lw = 2)
        plt.title('roi :{}, n_components : {} , cv_type : {}'.format(label, n_components, cv_type))
        plt.legend()
        plt.show()


    def get_index_cluster(self, list_labels, n_components): #for 1 roi 
        liste = []
        for i in range(n_components) : 
            liste.append(np.where(list_labels == i))

        return liste 

    def number_of_roi_after_model(self) : 
        number_of_roi = len(self.features)
        number_total_of_roi = 0
        for key in range(1, number_of_roi + 1) : 
            new_labels = self.get_index_cluster(self.parameters[key]['label'], self.parameters[key]['best_n_components'])
            number_total_of_roi += len(new_labels)

        return np.arange(1, number_total_of_roi+1)

    def new_clusters(self):
        number_of_roi = len(self.features)
        new_clusters = []
        for key in range(1, number_of_roi + 1) : 
            x,y,z = self.features[key]['coordonate']
            liste_new_labels = self.get_index_cluster(self.parameters[key]['label'], self.parameters[key]['best_n_components'])
            for liste in liste_new_labels : 
                subliste = []
                for index in liste[0] : 
                    subliste.append([x[index], y[index], z[index]])

                new_clusters.append(subliste)

        return new_clusters

    def new_labelled_mask(self):
        shape_labelled_mask = self.labelled_mask.shape
        labelled_mask = np.zeros((shape_labelled_mask[0], shape_labelled_mask[1], shape_labelled_mask[2]))
        liste_new_labels = self.number_of_roi_after_model()
        coordonate_new_clusters = self.new_clusters()
        for liste, label in zip(coordonate_new_clusters, liste_new_labels) :
            for index in liste : 
                labelled_mask[index[0], index[1], index[2]] = label

        return labelled_mask.astype(np.uint8)



    




import numpy as np

#with any threshold 3d mask label

def threshold_matrix(mask_array, pet_array, threshold):
    """ Return a thresholded mask
    """
    if threshold < 1 : 
        if len(mask_array.shape) != 3 :  #mask 4D 
            number_of_roi = mask_array.shape[3]
            liste = []
            for i in range(number_of_roi):
                new_mask = np.zeros((mask_array.shape[0],mask_array.shape[1], mask_array.shape[2] ))
                suv_values = []
                x,y,z = np.where(mask_array[:,:,:,i] != 0)
                if len(x) != 0 : 
                    suv_values = pet_array[x,y,z].tolist()
                    seuil = np.max(suv_values) * threshold
                    new_mask[np.where((pet_array > seuil) & (mask_array[:,:,:,i] > 0))] = 1
                    liste.append(new_mask)
                else : liste.append(new_mask)
            return np.stack(liste, axis = 3)    

        else : 
            maxi = np.max(mask_array)
            if maxi == 1.0 : 
                new_mask = np.zeros((mask_array.shape[0],mask_array.shape[1], mask_array.shape[2] ))
                x,y,z = np.where(mask_array != 0)
                suv_values = pet_array[x,y,z].tolist()
                seuil = np.max(suv_values) * threshold
                new_mask[np.where((pet_array > seuil) & (mask_array > 0))] = 1
                return new_mask
                 

            else :  
                number_of_roi = maxi
                new_mask = np.zeros((mask_array.shape[0],mask_array.shape[1], mask_array.shape[2] ))
                for i in range(number_of_roi):
                    suv_values = []
                    x,y,z = np.where(mask_array == i)
                    suv_values = pet_array[x,y,z].tolist()
                    seuil = np.max(suv_values) * threshold
                    new_mask[np.where((pet_array > seuil) & (mask_array == i))] = i
                return new_mask



    else : 
        if len(mask_array.shape) != 3 :  #mask 4D 
            number_of_roi = mask_array.shape[3]
            liste = []
            for i in range(number_of_roi):
                new_mask = np.zeros((mask_array.shape[0],mask_array.shape[1], mask_array.shape[2] ))
                seuil = threshold
                new_mask[np.where((pet_array > seuil) & (mask_array[:,:,:,i] > 0))] = 1
                liste.append(new_mask)

            return np.stack(liste, axis = 3)    

        else : 
            maxi = np.max(mask_array)
            if maxi == 1.0 : 
                new_mask = np.zeros((mask_array.shape[0],mask_array.shape[1], mask_array.shape[2] ))
                seuil = threshold
                new_mask[np.where((pet_array > seuil) & (mask_array > 0))] = 1
                return new_mask
                 

            else :  
                number_of_roi = maxi
                new_mask = np.zeros((mask_array.shape[0],mask_array.shape[1], mask_array.shape[2] ))
                seuil = threshold
                for i in range(number_of_roi):
                    new_mask[np.where((pet_array > seuil) & (mask_array == i))] = i
                return new_mask




"""
def get_threshold_matrix(ws_array, pet_array, number_of_roi, threshold) : #0.41
    for i in range(1, number_of_roi + 1):
        #print(i)
        points = []
        suv_values = []
        x,y,z = np.where(ws_array == i)
        for j in range(len(x)): 
            points.append([x[j], y[j], z[j]])

        #SEUIL A 41%
        for point in points : 
            suv_values.append(pet_array[point[0], point[1], point[2]])

        seuil = np.max(suv_values) * threshold


        for point in points : 
            if pet_array[point[0], point[1], point[2]] <= seuil : 
                ws_array[point[0], point[1], point[2]] = 0

    return ws_array

#for 4D matrix
def get_threshold_matrix_4D(mask, pet_array, threshold) : #0.41
    if len(mask.shape) != 3 :  #plusieurs ROI 
        number_of_roi = mask.shape[3]
    else : 
        number_of_roi = 1 #1 ROI 

    for i in range(number_of_roi):

        points = []
        suv_values = []
        if len(mask.shape) != 3 : #PLUSIEURS ROI 
            x,y,z = np.where(mask[:,:,:,i] != 0)
        else : #1 ROI 
            x,y,z = np.where(mask != 0)

        for j in range(len(x)): 
            points.append([x[j], y[j], z[j]])

        for point in points :  
            suv_values.append(pet_array[point[0], point[1], point[2]])

        if len(suv_values) != 0 :
            seuil = np.max(suv_values) * threshold
        else : seuil = 0


        for point in points : 
            if pet_array[point[0], point[1], point[2]] <= seuil : 
                if len(mask.shape) != 3 : #PLUSIEURS ROI 
                    mask[point[0], point[1], point[2], i] = 0
                else :  mask[point[0], point[1], point[2]] = 0 #1 ROI 

        
    return mask


#from csv

def get_suv_max(nifti_array, list_points):
    list_suv = []
    if list_points != [] :
        for point in list_points : 
            list_suv.append(nifti_array[point[1], point[0], point[2]])

        return np.max(list_suv)
    else : return 0.0 #pas de suv max


def threshold_mask(mask_4D, details_rois, nifti_array):

    number_of_roi = mask_4D.shape[3]

    for roi in range(number_of_roi) : 
        list_points = details_rois[roi + 1]['list_points']
        suv_max = get_suv_max(nifti_array, list_points)

            #GET THRESHOLD
        threshold = details_rois['SUVlo']
        if "%" in threshold : 
            threshold = float(threshold.strip("%"))/100 * suv_max
        else : 
            threshold = float(threshold)
        

        for point in list_points :
            if nifti_array[point[1], point[0], point[2]] <= threshold :

                mask_4D[point[1], point[0], point[2], roi] = 0
 
    return mask_4D


""" 
            
       

            

            


import pydicom
import os
import numpy as np

from library_dicom.dicom_processor.enums.TagEnum import *
from library_dicom.dicom_processor.enums.SopClassUID import *

class Instance:
    """A class to represent a Dicom file 
    """

    def __init__(self, path, load_image=True):
        """Construct a Dicom file object

        Arguments:
            path {[String]} -- [Absolute path where the Dicom file is located]
        """
        self.path = path
        if (load_image) : self.__load_full_instance()
        else : self.__load_metadata()

    def __load_metadata(self):
        self.dicomData = pydicom.dcmread(self.path, stop_before_pixels=True, force=True)
    
    def __load_full_instance(self):
        self.dicomData = pydicom.dcmread(self.path, force=True)
  
    def get_series_tags(self):
        series_tags={}
        for tag_address in TagsSeries:
            if tag_address.value in self.dicomData :
                series_tags[tag_address.name] = self.dicomData[tag_address.value].value
            else : series_tags[tag_address.name] = "Undefined"
        series_tags[ImageType.ImageType.name] = self.get_image_type()
        series_tags[PixelSpacing.PixelSpacing.name] = self.get_pixel_spacing()
        return series_tags

    def get_patients_tags(self):
        patient_tags={}
        for tag_address in TagsPatient:
            if tag_address.value in self.dicomData : 
                patient_tags[tag_address.name] = self.dicomData[tag_address.value].value
                if( type(self.dicomData[tag_address.value].value) != str):
                    #patient name return PersonName3 object, convert it to string
                    patient_tags[tag_address.name] = str(patient_tags[tag_address.name])
            else : patient_tags[tag_address.name] = "Undefined"
        return patient_tags

    def get_studies_tags(self):
        studies_tags={}
        for tag_address in TagsStudy:
            if tag_address.value in self.dicomData : studies_tags[tag_address.name] = self.dicomData[tag_address.value].value
            else : studies_tags[tag_address.name] = "Undefined"
        return studies_tags

    def get_instance_tags(self):

        instance_tags={}
        instance_tags['PixelSpacing'] = self.get_pixel_spacing()
        instance_tags['ImagePosition'] = self.get_image_position()
        instance_tags['ImageOrientation'] = self.get_image_orientation()
        instance_tags['RescaleSlope'] = self.get_rescale_slope()
        instance_tags['RescaleIntercept'] = self.get_rescale_intercept()
        instance_tags['SOPInstanceUID'] = self.get_SOPInstanceUID()
        return instance_tags

    def get_sop_class_uid(self):
        if 'SOPClassUID' in self.dicomData.dir() : return self.dicomData.SOPClassUID
        else : raise Exception('Undefined SOP Class UID')

    def get_radiopharmaceuticals_tags(self):
        radiopharmaceuticals_tags={}
        radiopharmaceutical_sequence = []

        try :
            radiopharmaceutical_sequence = self.dicomData[0x00540016][0]
        except Exception: 
            print("no Radiopharmaceuticals tags")

        for tag_address in TagsRadioPharmaceuticals:
            if tag_address.value in radiopharmaceutical_sequence : radiopharmaceuticals_tags[tag_address.name] = radiopharmaceutical_sequence[tag_address.value].value
            else : radiopharmaceuticals_tags[tag_address.name] = "Undefined"
            
        return radiopharmaceuticals_tags

    def get_pet_correction_tags(self):
        if TagPTCorrection.CorrectedImage.value in self.dicomData :
            return list(self.dicomData[TagPTCorrection.CorrectedImage.value].value)
        else: return "Undefined"


    def get_philips_private_tags(self):
        philips_tags={}
        for tag_address in PhilipsPrivateTags:
            if tag_address.value in self.dicomData : 
                philips_tags[tag_address.name] = float(self.dicomData[tag_address.value].value)
            else : philips_tags[tag_address.name] = "Undefined"
        return philips_tags

    def is_secondary_capture(self):
        return True if self.get_sop_class_uid in CapturesSOPClass else False

    def get_SOPInstanceUID(self):
        return self.dicomData[TagsInstance.SOPInstanceUID.value].value

    def get_rescale_slope(self):
        return self.dicomData[TagsInstance.RescaleSlope.value].value

    def get_rescale_intercept(self):
        return self.dicomData[TagsInstance.RescaleIntercept.value].value

    def get_image_orientation(self):
        return list(self.dicomData[TagsInstance.ImageOrientation.value].value)
    
    def get_image_position(self):
        return list(self.dicomData[TagsInstance.ImagePosition.value].value)

    def get_pixel_spacing(self):
        return list(self.dicomData[PixelSpacing.PixelSpacing.value].value)    
        

    def get_image_type(self):
        return list(self.dicomData[ImageType.ImageType.value].value)

    def is_image_modality(self):
        sop_values = set(item.value for item in ImageModalitiesSOPClass)
        return True if self.get_sop_class_uid() in sop_values else False

    def get_image_nparray(self):
        if self.is_image_modality() == False : 
            raise Exception('Not Image Modality')
        else:
            pixel_array = self.dicomData.pixel_array
            rescale_slope = self.get_rescale_slope()
            rescale_intercept = self.get_rescale_intercept()

            resultArray = ( pixel_array * rescale_slope) + rescale_intercept
            return resultArray
    

    def get_series_instance_uid(self):
        return self.dicomData[TagsSeries['SeriesInstanceUID'].value].value

    def get_number_rows(self):
        return self.dicomData.Rows

    def get_number_columns(self):
        return self.dicomData.Columns


    def get_acquisition_date(self):
        if "AcquisitionDate" in self.dicomData : 
            return self.dicomData.AcquisitionDate
        else : return "Undefined"
        

    def get_acquisition_time(self):
        if 'AcquisitionTime' in self.dicomData : 
            return self.dicomData.AcquisitionTime
        else : return "Undefined"


    #for RTSS Writer 
    #check si ces informations sont bien dans le dataset sinon erreur 

    def get_accession_number(self):
        return self.dicomData[TagsStudy['AccessionNumber'].value].value 

    def get_patient_name(self):
        return self.dicomData[TagsPatient['PatientName'].value].value

    def get_patient_id(self):
        return self.dicomData[TagsPatient['PatientID'].value].value

    def get_patient_birth_date(self):
        return self.dicomData[TagsPatient['PatientBirthDate'].value].value

    def get_patient_sex(self):
        return self.dicomData[TagsPatient['PatientSex'].value].value 

    def get_study_date(self):
        return self.dicomData[TagsStudy['StudyDate'].value].value

    def get_study_description(self):
        return self.dicomData[TagsStudy['StudyDescription'].value].value 

    def get_study_id(self):
        return self.dicomData[TagsStudy['StudyID'].value].value 

    def get_study_instance_uid(self):
        return self.dicomData[TagsStudy['StudyInstanceUID'].value].value

    def get_study_time(self):
        return self.dicomData[TagsStudy['StudyTime'].value].value

    def get_referring_physician_name(self):
        if "ReferringPhysicianName" in self.dicomData : return self.dicomData.ReferringPhysicianName
        else : return "Undefined"

    def get_specific_character_set(self):
        if "SpecificCharacterSet" in self.dicomData : return self.dicomData.SpecificCharacterSet
        else : return "Undefined"

    def get_physicians_of_record(self):
        if "PhysiciansOfRecord" in self.dicomData : return self.dicomData.PhysiciansOfRecord
        else : return "Undefined"

    
    def get_frame_of_reference_uid(self):
        return self.dicomData.FrameOfReferenceUID 




        


    




    

    


import json
import os
import shutil
import pprint
from collections import defaultdict, Counter

def generate_merged_file(json_path):
    filenames = os.listdir(json_path)
    content_map = defaultdict(dict)

    for filename in filenames:
        with open(os.path.join(json_path, filename)) as json_file:
            data = json.load(json_file)
            patientID = str(data['patient']['PatientID'])
            studyUID = str(data['study']['StudyInstanceUID'])
            studyDetails = data['study']
            seriesDetails = data['series']
            seriesInstanceUID = data['series']['SeriesInstanceUID']

            if(studyUID not in content_map[patientID]) :
                content_map[patientID][studyUID]=studyDetails
            
            if('Series' not in content_map[patientID][studyUID] ):
                content_map[patientID][studyUID]['Series']={}
            content_map[patientID][studyUID]['Series'][seriesInstanceUID]=seriesDetails
            content_map[patientID][studyUID]['Series'][seriesInstanceUID]['path'] = data['path']
            content_map[patientID][studyUID]['Series'][seriesInstanceUID]['files'] = data['files']

    return content_map

def find_non_intersting_series(json_merged_file_path):
    paths = []
    data = json.load(open(json_merged_file_path))
    for patientID in data:
        for studyUID in data[patientID]:
            for seriesUID in data[patientID][studyUID]['Series']:
                path = data[patientID][studyUID]["Series"][seriesUID]["path"]
                series_description = data[patientID][studyUID]["Series"][seriesUID]["SeriesDescription"]
                non_interesting_key_word =("CT COUPES FINES", "_WB_NAC", "POUMON", "PARENCHYME", "ORL", "LUNG", "COMPLEMENTAIRE"
                , "EARL", "MEMBRE", "CORO", "SAG")
                for key_word in non_interesting_key_word:
                    if(key_word in series_description.upper() ):
                        paths.append( path )
    return paths

def find_studies_over_two_series(json_merged_file_path):
    data = json.load(open(json_merged_file_path))
    number = 0
    series_to_check=[]
    series_list = []
    paths = []
    for patientID in data:
        for studyUID in data[patientID]:
            if( len(data[patientID][studyUID]['Series']) >2 ):
                number +=1
                for seriesUID in data[patientID][studyUID]['Series']:
                    path = data[patientID][studyUID]["Series"][seriesUID]["path"]
                    paths.append(path)
                    series_list.append(data[patientID][studyUID]['Series'][seriesUID]['SeriesDescription'])
                    series_to_check.append(data[patientID][studyUID]['Series'])
    #print(series_list)
    return series_list, paths


def find_studies_with_two_series(json_merged_file_path):
    data = json.load(open(json_merged_file_path))
    matching_series = defaultdict(dict)
    paths = []
    for patientID in data:
        for studyUID in data[patientID]:
            if( len(data[patientID][studyUID]['Series']) == 2 ):
                for seriesUID in data[patientID][studyUID]['Series']:
                    matching_series[seriesUID]['seriesDetails'] = data[patientID][studyUID]['Series'][seriesUID]
                    matching_series[seriesUID]['path'] = data[patientID][studyUID]['Series'][seriesUID]['path']
                    matching_series[seriesUID]['parentStudyUID'] = studyUID
                    matching_series[seriesUID]['parentPatientID'] = patientID
                    paths.append(data[patientID][studyUID]['Series'][seriesUID]['path'])
    #pp = pprint.PrettyPrinter(depth=6)
    #pp.pprint(matching_series)
    return matching_series, paths

def remove_path_from_disk(path):
    try:
        shutil.rmtree(path)
    except Exception as err:
        print(err)


import os
from library_dicom.dicom_processor.model.reader.Instance import Instance
from library_dicom.dicom_processor.model.Series import Series
from library_dicom.dicom_processor.model.SeriesPT import SeriesPT
from library_dicom.dicom_processor.enums.SopClassUID import *

def get_series_object(path):
    try:
        first_file_name = os.listdir(path)[0]
    except Exception as err:
        print(err)
        print(path)
    first_instance = Instance( os.path.join(path,first_file_name) )
    sop_class_uid = first_instance.get_sop_class_uid()
    if(sop_class_uid == ImageModalitiesSOPClass.PT.value or sop_class_uid == ImageModalitiesSOPClass.EnhancedPT.value):
        return SeriesPT(path)
    else : return Series(path)



from library_dicom.dicom_processor.model.Series import Series

from library_dicom.export_segmentation.rtstruct.StructureSetROISequence import StructureSetROISequence
from library_dicom.export_segmentation.rtstruct.RTROIObservationsSequence import RTROIObservationsSequence
from library_dicom.export_segmentation.rtstruct.ROIContourSequence import ROIContourSequence
from library_dicom.export_segmentation.rtstruct.ReferencedFrameOfReferenceSequence import ReferencedFrameOfReferenceSequence
from library_dicom.export_segmentation.tools.generate_dict import *
from library_dicom.export_segmentation.tools.rtss_writer_tools import *
import pydicom
import datetime
import random 
import warnings
import os 
import numpy as np 
import tempfile 

#DOCU : 
# MASK : NDARRAY 3D [Z, X, Y] OR SITK IMG (X, Y, Z) => CHAQUE SLICE NE DOIT PAS AVOIR MOINS DE 3 PIXELS ISOLES
#(cf clean argument)
#DICT : generate_dict in tools : PAS PLUS DE 16 CARACTERES 
class RTSS_Writer:
    """A class for DICOM RT format
    """
    
    def __init__(self, mask, serie_path, mask_mode, clean = True, sous_seg = True):
        """[summary]

        Args:
            mask ([ndarray or sitk_img]): []
            serie_path ([str]): [Serie path related to RTSTRUCT file ]
            mask_mode ([str]) : ['img' if a sitk_img or 'matrix' if ndarray]
            clean ([bool]) : [Clean the mask or not. Set to True]
            label ([bool]) : [Transform binary mask into labelled mask. Set to True]
        """

        self.mask = mask
        self.mask_mode = mask_mode 
        self.clean = clean 
        self.sous_seg = sous_seg

        serie = Series(serie_path)
        self.instances = serie.get_instances_ordered()


        if mask_mode == 'matrix' : 
            self.mask_array = mask # 3D ndarray [z, x, y]
            self.mask_img = sitk.GetImageFromArray(self.mask_array)
            #data spécifique à la série ou on dessine les contours : origin, spacing, direction 

            self.image_position = self.instances[0].get_image_position() #origin 
            self.pixel_spacing = self.instances[0].get_pixel_spacing()
            self.pixel_spacing.append(abs(serie.get_z_spacing())) #spacing 
            original_direction = self.instances[0].get_image_orientation()
            self.image_direction = (float(original_direction[0]), float(original_direction[1]), float(original_direction[2]), 
                                        float(original_direction[3]), float(original_direction[4]), abs(float(original_direction[5])), 
                                        0.0, 0.0, 1.0) 
            self.mask_img.SetSpacing(self.pixel_spacing)
            self.mask_img.SetOrigin(self.image_position)
            self.mask_img.SetDirection(self.image_direction)

        elif mask_mode == 'img' : 
            self.mask_img = mask
            self.mask_array = read_sitk_img(self.mask_img) # 3Dndarray [z,x,y]
            self.image_position = self.mask_img.GetOrigin()
            self.pixel_spacing = self.mask_img.GetSpacing()
            self.image_direction = self.mask_img.GetDirection()

        if clean == True : 
            self.mask_array = clean_mask(self.mask_array)

        if sous_seg == True : 
            #Ici implémenter watershed 
            self.mask_array = label(self.mask_array, connectivity=2)



        #get number of ROI after label
        self.number_of_roi = int(np.max(self.mask_array))



        #Get list of every sop instance uid 
        self.list_all_SOPInstanceUID = get_list_SOPInstance_UID(self.instances)

        #generate dictionnary with paramter inside 
        self.generate_dict_json()

        #creation file_meta 
        self.file_meta, self.file_meta.MediaStorageSOPInstanceUID = self.generates_file_meta()
        suffix = '.dcm'
        filename_little_endian = tempfile.NamedTemporaryFile(suffix=suffix).name
        self.dataset = pydicom.dataset.FileDataset(filename_little_endian, {}, file_meta=self.file_meta, preamble=b"\0" * 128)

        #Add the data element in FileDataset
        self.set_tags()
        self.set_StructureSetROISequence()
        self.set_RTROIObservationSequence()
        self.set_ROIContourSequence()
        self.set_ReferencedFrameOfReferenceSequence()
        
        # Set the transfer syntax
        self.dataset.is_little_endian = True
        self.dataset.is_implicit_VR = False 

    def generate_dict_json(self):
        number_of_roi = get_number_of_roi(self.mask_array)
        results = generate_dict(number_of_roi, 'rtstruct')
        self.results = results
        return None 

    def generates_file_meta(self):
        """
            Generates required values for file meta information
            List of tags :
                  - FileMetaInformationGroupLength':
                  - FileMetaInformationVersion'    :
                  - MediaStorageSOPClassUID'       :
                  - MediaStorageSOPInstanceUID'    :
                  - TransferSyntaxUID'             :
                  - ImplementationClassUID'        :
        """


        file_meta = pydicom.dataset.FileMetaDataset()

        file_meta.FileMetaInformationGroupLength = 166
        file_meta.FileMetaInformationVersion = b'\x00\x01'
        file_meta.MediaStorageSOPClassUID = '1.2.840.10008.5.1.4.1.1.481.3' # RT Structure Set Storage
        file_meta.MediaStorageSOPInstanceUID = pydicom.uid.generate_uid()
        #self.dataset.SOPInstanceUID = file_meta.MediaStorageSOPInstanceUID 
        file_meta.TransferSyntaxUID = pydicom.uid.ExplicitVRLittleEndian #pydicom.uid.ImplicitVRLittleEndian #pydicom.uid.ExplicitVRLittleEndian  #Implicit VR Little Endian
        file_meta.ImplementationClassUID =  pydicom.uid.PYDICOM_IMPLEMENTATION_UID #'1.2.246.352.70.2.1.7'
        
        return file_meta, file_meta.MediaStorageSOPInstanceUID


    def set_tags(self):
        """
            
            List of tags :
                  - AccessionNumber'       :
                  - PatientBirthDate'      :
                  - PatientBirthTime'      :
                  - PatientID'             :
                  - PatientName'           :
                  - PatientSex'            :
                  - PhysiciansOfRecord'    :
                  - ReferringPhysicianName':
                  - SpecificCharacterSet'  :
                  - StudyDate'             :
                  - StudyDescription'      :
                  - StudyID'               :
                  - StudyInstanceUID'      :
                  - StudyTime'             : 
        """     
        #from the serie 
        self.dataset.AccessionNumber = self.instances[0].get_accession_number()
        self.dataset.PatientBirthDate = self.instances[0].get_patient_birth_date()
        self.dataset.PatientID = self.instances[0].get_patient_id()
        self.dataset.PatientName = self.instances[0].get_patient_name()
        self.dataset.PatientSex = self.instances[0].get_patient_sex()
        self.dataset.PhysiciansOfRecord = self.instances[0].get_physicians_of_record()
        self.dataset.ReferringPhysicianName = self.instances[0].get_referring_physician_name()
        self.dataset.SpecificCharacterSet = self.instances[0].get_specific_character_set()
        self.dataset.StudyDate = self.instances[0].get_study_date()
        self.dataset.StudyDescription = self.instances[0].get_study_description()
        self.dataset.StudyID = self.instances[0].get_study_id()
        self.dataset.StudyInstanceUID = self.instances[0].get_study_instance_uid()
        self.dataset.StudyTime = self.instances[0].get_study_time()

        #specific new tags for the serie 
        self.dataset.ApprovalStatus = 'UNAPPROVED'
        self.dataset.Manufacturer   = ''
        dt = datetime.datetime.now()
        self.dataset.InstanceCreationDate = dt.strftime('%Y%m%d')
        self.dataset.InstanceCreationTime = dt.strftime('%H%M%S.%f')
        self.dataset.InstanceNumber = '1'
        self.dataset.Modality = 'RTSTRUCT'
        self.dataset.ReviewDate = '' #because UNAPPROVED
        self.dataset.ReviewTime = '' #because UNAPPROVED
        self.dataset.ReviewerName = '' #because UNAPPROVED
        self.dataset.SeriesDescription = self.results["SeriesDescription"]
        self.dataset.SeriesInstanceUID = pydicom.uid.generate_uid()
        self.dataset.SeriesNumber = random.randint(0,1e3)
        self.dataset.SOPClassUID = '1.2.840.10008.5.1.4.1.1.481.3' 
        self.dataset.SOPInstanceUID = self.file_meta.MediaStorageSOPInstanceUID
        self.dataset.StructureSetDate = dt.strftime('%Y%m%d')
        self.dataset.StructureSetDescription = self.results["SeriesDescription"]
        self.dataset.StructureSetLabel = self.results["SeriesDescription"]
        self.dataset.StructureSetTime = dt.strftime('%H%M%S.%f')

        return None 



    #StructureSetROISequence
    def set_StructureSetROISequence(self):
        referenced_frame_of_reference_uid = self.instances[0].get_frame_of_reference_uid()
        self.dataset.StructureSetROISequence = StructureSetROISequence(self.mask_array, self.results, self.number_of_roi).create_StructureSetROISequence(self.pixel_spacing, referenced_frame_of_reference_uid)
        

    #RTROIObservationSequence
    def set_RTROIObservationSequence(self):
        self.dataset.RTROIObservationsSequence = RTROIObservationsSequence(self.results, self.number_of_roi).create_RTROIObservationsSequence()
        

    #ROIContourSequence 
    def set_ROIContourSequence(self):
        referenced_sop_class_uid = self.instances[0].get_sop_class_uid()
        self.dataset.ROIContourSequence = ROIContourSequence(self.mask_array, self.mask_img, self.number_of_roi).create_ROIContourSequence(referenced_sop_class_uid, self.list_all_SOPInstanceUID, self.instances)

    #ReferencedFrameOfReferenceSequence 
    def set_ReferencedFrameOfReferenceSequence(self):
        frame_of_reference_uid = self.instances[0].get_frame_of_reference_uid()
        series_instance_uid = self.instances[0].get_series_instance_uid()
        study_instance_uid = self.instances[0].get_study_instance_uid()
        sop_class_uid = self.instances[0].get_sop_class_uid()
        self.dataset.ReferencedFrameOfReferenceSequence = ReferencedFrameOfReferenceSequence().create_ReferencedFrameOfReferenceSequence(frame_of_reference_uid, sop_class_uid, self.list_all_SOPInstanceUID, series_instance_uid, study_instance_uid)


    def save_file(self, filename, directory_path):
        #filemeta = self.generates_file_meta()
        #filedataset = pydicom.dataset.FileDataset(filename, self.dataset, preamble=b"\0" * 128, file_meta=filemeta, is_implicit_VR = True, is_little_endian = True )
        #filedataset.save_as(os.path.join(directory_path, filename))
        self.dataset.save_as(os.path.join(directory_path, filename), write_like_original=False)
        return None 


        
        

    



from library_dicom.dicom_processor.enums.SopClassUID import *
import SimpleITK as sitk
import numpy as np

class NiftiBuilder:
    """BuildNifti File using a Series Object
    """

    def __init__(self, series):
        self.series = series

    
    def save_nifti(self, filename, mask = None):
        if( mask is None) : 
            sitk_img = sitk.GetImageFromArray( np.transpose(self.series.get_numpy_array(), (2,0,1) ))
            original_pixel_spacing = self.series.instance_array[0].get_pixel_spacing()
            
            original_direction = self.series.instance_array[0].get_image_orientation()
            sitk_img.SetDirection( (float(original_direction[0]), float(original_direction[1]), float(original_direction[2]), 
                                    float(original_direction[3]), float(original_direction[4]), float(original_direction[5]), 
                                    0.0, 0.0, 1.0) )
            sitk_img.SetOrigin( self.series.instance_array[0].get_image_position() )
            sitk_img.SetSpacing( (original_pixel_spacing[0], original_pixel_spacing[1], self.series.get_z_spacing()) )
            sitk.WriteImage(sitk_img, filename)
            
        else :
            number_of_roi = mask.shape[3]
            slices = []
            for number_roi in range(number_of_roi):

                sitk_img = sitk.GetImageFromArray( np.transpose(mask[:,:,:,number_roi], (2,0,1) ))
        
                original_pixel_spacing = self.series.instance_array[0].get_pixel_spacing()
                original_direction = self.series.instance_array[0].get_image_orientation()
                sitk_img.SetDirection( (float(original_direction[0]), float(original_direction[1]), float(original_direction[2]), 
                                    float(original_direction[3]), float(original_direction[4]), float(original_direction[5]), 
                                    0.0, 0.0, 1.0) )
                sitk_img.SetOrigin( self.series.instance_array[0].get_image_position() )
                sitk_img.SetSpacing( (original_pixel_spacing[0], original_pixel_spacing[1], self.series.get_z_spacing()) )
                slices.append(sitk_img)
            
            img = sitk.JoinSeries(slices)
            sitk.WriteImage(img, filename)




from library_dicom.dicom_processor.model.Series import Series
from library_dicom.dicom_processor.model.reader.Instance import Instance
from library_dicom.dicom_processor.model.reader.Instance_RTSS import Instance_RTSS
import pydicom
import os
import cv2 as cv2
import numpy as np 
import sys 


class MaskBuilder_RTSS : 

    def __init__(self, rtss_path, serie_path):

        self.serie = Series(serie_path)
        self.serie_data = self.serie.get_series_details()
        self.instance_uid_serie = self.serie.get_all_SOPInstanceIUD()
        self.matrix_size = self.serie.get_size_matrix()
        self.instances = self.serie.get_instances_ordered()
        
        #to recenter spatial coordonate 
        self.image_position = self.instances[0].get_image_position()
        self.pixel_spacing = self.instances[0].get_pixel_spacing()
        self.pixel_spacing.append(self.serie.get_z_spacing())

        #print(self.image_position)
        #print(self.pixel_spacing)

        self.rtss = Instance_RTSS(rtss_path)
        self.number_of_roi = self.rtss.get_number_of_roi()

    def get_list_all_SOP_Instance_UID_serie(self):
        return self.instance_uid_serie


    def is_sop_instance_uid_same(self):
        uid_serie = self.get_list_all_SOP_Instance_UID_serie()
        uid_rtss = self.rtss.get_list_all_SOP_Instance_UID_RTSS()
        for uid in uid_rtss : 
            if uid not in uid_serie : 
                return False

        return True

    def get_image_position_per_slice(self):
        liste = []
        for i in range(len(self.instances)):
            liste.append(self.instances[i].get_image_position())
        return liste


    """
    def get_image_position(self):
        #print("origin : ", self.serie_data['instance']['ImagePosition'] )
        return self.serie_data['instance']['ImagePosition']

    def get_pixel_spacing(self):
        pixel_spacing =  self.serie_data['instance']['PixelSpacing']
        instances = self.serie.get_instances_ordered()
        pixel_spacing.append(self.serie.get_z_spacing())
        for i in range(len(instances)):
            print(instances[i].get_image_position())
        return pixel_spacing
    """ 


    def __spatial_to_pixels(self, matrix_size, number_roi, list_all_SOPInstanceUID):
        """Transform contour data in spatial to contour data in pixels, return a dict 


        Returns:
            [dict] -- [{1 : { x : []
                               y : []
                               z : []} , 
                       {2 : { x : []
                               y : []
                               z : []} , ...]
        """
        #frame_of_reference_UID = self.get_frame_of_reference_UID()

        #pixel_spacing = self.get_pixel_spacing()
        #image_position = self.get_image_position()

        #list_referenced_SOP_instance_uid = self.rtss.get_list_referenced_SOP_Instance_UID(number_roi)
        
        list_contour_data = self.rtss.get_contour_data(number_roi) #x y z en mm 
        list_pixels = {}
        for i in range(3):
            self.image_position[i] = float(self.image_position[i])

        for contour_data in (list_contour_data):
            number_item = list_contour_data.index(contour_data) #0 1 2 3 ...
            contour_item = {}
            x = contour_data[::3]  #list
            x = [int(round((i - self.image_position[0]) / self.pixel_spacing[0] )) for i in x ] 
            #x = [int((i / self.pixel_spacing[0] )) for i in x ]
            contour_item['x'] = x
            y = contour_data[1::3] #list
            y = [int(round((i - self.image_position[1]) / self.pixel_spacing[1] ))  for i in y ]
            #y = [int((i  / self.pixel_spacing[1] ))  for i in y ]  
            contour_item['y'] = y

            z = contour_data[2::3]
            #print(z)
            z = [int(round((i - self.image_position[2]) / abs(self.pixel_spacing[2]) )) for i in z ][0]
            #print(z)
            #z = [int((i  / self.pixel_spacing[2] )) for i in z ]
            #z = list_referenced_SOP_instance_uid[number_item]
            #z = list_all_SOPInstanceUID.index(z) #numero de coupe correspondant
            contour_item['z'] = z
            list_pixels[number_item + 1] = contour_item


        return list_pixels

    #eventuellement en privé
    def get_list_points(self, matrix_size, number_roi, list_all_SOPInstanceUID):
        """transform a list of pixels of a ROI to a list nx2 (n points, coordonate (x,y)) for each contour

        Arguments:
          

        Returns:
            [list] -- list of (x,y) points and list of z slices in which there is a contour
        """
        pixels = self.__spatial_to_pixels(matrix_size, number_roi, list_all_SOPInstanceUID) #dict 
        number_item = len(pixels)
        list_points = []
        slice = []
        for item in range(number_item):
            subliste = []
            list_x = (pixels[item + 1]['x'])
            list_y = (pixels[item + 1]['y'])
            for x,y in zip(list_x, list_y):
                subliste.append([x,y])

            list_points.append(subliste)

            slice.append(pixels[item + 1]['z'])

        return list_points, slice 

  
    def rtss_to_3D_mask(self, number_roi, matrix_size, list_all_SOPInstanceUID):
        number_of_slices = matrix_size[2]
        np_array_3D = np.zeros(( matrix_size[0],  matrix_size[1], number_of_slices)).astype(np.uint8)
        if self.rtss.is_closed_planar(number_roi) == False : raise Exception ("Not CLOSED_PLANAR contour")
        liste_points, slice = self.get_list_points(matrix_size, number_roi, list_all_SOPInstanceUID )
        #print(slice)
        for item in range(len(slice)):
            np_array_3D[:,:,slice[item]] = cv2.drawContours(np.float32(np_array_3D[:,:,slice[item]]), [np.asarray(liste_points[item])], -1, number_roi , -1)

        return np_array_3D.astype(np.uint8)


    def rtss_to_4D_mask(self):
        matrix_size = self.matrix_size
        list_all_SOPInstanceUID = self.rtss.get_list_all_SOP_Instance_UID_RTSS()
        np_array_3D = []
        for number_roi in range(1, self.number_of_roi +1):
            np_array_3D.append(self.rtss_to_3D_mask(number_roi, matrix_size, list_all_SOPInstanceUID))
        np_array_4D = np.stack((np_array_3D), axis = 3)

        return np_array_4D.astype(np.uint8)


from library_dicom.dicom_processor.model.reader.Instance_RTDose import Instance_RTDose
import matplotlib.pyplot as plt 
import plotly_express as px 
import pandas as pd 
import numpy as np 


def convert_str_list_to_float_list(str_liste):
    empty_list = []
    for i in range(len(str_liste)):
        empty_list.append(float(str_liste[i]))
    return empty_list 


def generate_x_y_cumul(liste):
    x = []
    y = []
    somme = 0
    for i in range(len(liste)) : 
        if i%2 == 1 : 
            y.append(liste[i])
        else : 
            somme += liste[i]
            x.append(somme)
    return x,y 

def generate_plot(dictionnary, mode = 'cumulative', dose_mode = 'Gray', volume_mode = '%'):
    """[summary]

    Args:
        dictionnary ([type]): [description]
        mode (str, optional): ['cumulative' or 'differentiel' only]. Defaults to 'cumulative'.
        dose_mode (str, optional): ['%' or 'Gray' only ]. Defaults to 'Gray'.
        volume_mode (str, optional): ['%' or 'cc' only ]. Defaults to '%'.

    Returns:
        [type]: [description]
    """
    keys = list(dictionnary.keys())
    dataframe  = []
    for key in keys : 
        data_str = dictionnary[key]['DVHData']
        name = dictionnary[key]['ReferencedROIName']
        data_float = convert_str_list_to_float_list(data_str)
        x,y = generate_x_y_cumul(data_float)


        if mode == 'cumulative' : 

            #volume
            if volume_mode == '%' : 
                new_y = []
                maxi = np.max(y)
                for vol in y : 
                    new_y.append(vol / maxi * 100)
            elif volume_mode == 'cc' : 
                new_y = y 
            else : print('Mode not available')


            #dose 
            if dose_mode == 'Gray' : 
                new_x = x 
            elif dose_mode == '%' : 
                new_x = []
                maxi = np.max(x)
                for rayon in x : 
                    new_x.append(rayon/maxi * 100 )
            else : print('Mode not available')


            for i in range(len(new_x)):
                subliste = []
                subliste.append(name)
                subliste.append(new_x[i])
                subliste.append(new_y[i])
                dataframe.append(subliste)

        if mode == 'differentiel' : 
            #conversion volume to differentiel 
            differentiel = []
            y.reverse()
            differentiel.append(y[0])
            for i in range(1, len(y)):
                differentiel.append(abs(y[i] - y[i-1]))

            #volume 
            if volume_mode == '%' : 
                new_y = []
                maxi = np.max(differentiel)
                for vol in differentiel : 
                    new_y.append(vol / maxi * 100)
            elif volume_mode == 'cc' : 
                new_y = differentiel
            else : print('Mode not available')

            new_y.reverse()

            #dose 
            if dose_mode == 'Gray' : 
                new_x = x 
            elif dose_mode == '%' : 
                new_x = []
                maxi = np.max(x)
                for rayon in x : 
                    new_x.append(rayon/maxi * 100 )
            else : print('Mode not available')

            for i in range(len(new_x)):
                subliste = []
                subliste.append(name)
                subliste.append(new_x[i])
                subliste.append(new_y[i])
                dataframe.append(subliste)


    df = pd.DataFrame(dataframe, columns=['ROIName', "dose_{}".format(dose_mode), "volume_{}".format(volume_mode)])
    fig = px.line(df, x="dose_{}".format(dose_mode), y="volume_{}".format(volume_mode), color='ROIName', title=mode, width = 750, height = 550)
    fig.show()

    return None 

    

